//************************ 2608vi.c ***************************
//* This file contains Lab View DLL to communicate with
//  Sensoray model 2608 analog i/o board.
//* Written by Alexander Kostromitin
//* Copyright (C) Sensoray Company 2006
//*************************************************************


#define DEBUG     1     // generate some additional info


#include <time.h>
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include <commdlg.h>    // includes common dialog functionality
#include <commctrl.h>
#include <dlgs.h>       // includes common dialog template defines
#include <cderr.h>      // includes the common dialog error codes
#include <io.h>
#include <math.h>


//#include "Win2600.h"
#include "f2600.h"

static int dllinit(void);
static void dllclose(void);
static int findmodule(int mm, int id, int from);
static void getinternals(int mm, int mod);

static int io_exec(void *x);

static void error(char *fmt, ... );


#define NUM_ADC     16          // number of input channels
#define NUM_DAC     8           // number of output channels

#define ADC_RANGE   32768       // +32767(0x7fff)/-32768(0x8000)
#define DAC_RANGE   32768       // +32767(0x7fff)/-32768(0x8000)


// Common parameters
static char errstr[256];                // error message string


#define NUM_ADC         16
#define NUM_DAC         8
#define NUM_TSENSORS    8

// In/out parameters
static char intype[NUM_ADC] =       // input types
        {
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE,
        TC_J_TYPE
        };

static double adcval[NUM_ADC];             // ADC input values
static double reftemps[NUM_ADC];           // reference temperatures

static short adcinternals[12];             // internal (standard) inputs




// DLL entry point
BOOL WINAPI DllEntryPoint(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}

// alternative entry point
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}




/////////////////////////////////////////////////////////////////
// exported functions
//
// note: for compatibility reasons all exported functions
// declared as __stdcall (WINAPI), do not use regular C
// convention to call them from VI.
/////////////////////////////////////////////////////////////////

// Open board
//  ClientDotAdrs: IP address of host computer's connection (use "0" for default)
//  MMDotAdrs: IP address of Main Module (2601)
//  mm: index of registering Main Module in internal DLL's table
//  iom: module (2601 IOM connector) number (use -1 to open first module found)
//  wdmsec: watcdog interval in milliseconds (maximum time between commands to
//   avoid internal reset; use 0 to disable)
__declspec (dllexport) int __stdcall openboard(char *ClientDotAdrs, char *MMDotAdrs, int mm, int iom, int wdmsec)
{
  int err;
  void *x;
  int port;

  // establish connection with 2601 base module
  S26_BoardOpen(mm, ClientDotAdrs, MMDotAdrs);
  err = !S26_ResetNetwork(mm);
  if(err)
    {
    error("Can't initialize the MM");
    return(-1);
    }
  // set watchdog
  if(wdmsec != 0)
    {
    wdmsec /= 100;
    if(wdmsec <= 0)
      wdmsec = 1;
    else if(wdmsec > 255)
      wdmsec = 255;
    }
  x = S26_SchedOpen(mm, 1);
  S26_Sched2601_SetWatchdog(x, wdmsec);
  io_exec(x);

  // find 1st board
  port = findmodule(mm, 2608, iom);
  if(port < 0)
    {
    error("No board found");
    return(-1);
    }
  return(port);
}


// Refresh internal calibration values and get internal temperatures
//  iom: module number
__declspec (dllexport) double __stdcall getreftemp(int mm, int iom, int cnl)
{
  int icnt, tcnt;
  int lg0;
  int lg10;

  if(cnl < 0 || cnl >= NUM_ADC)
    cnl = 0;
  getinternals(mm, iom);
  lg0 = adcinternals[0];
  lg10 = adcinternals[1];
  // interpolate results from 8 sensors to 16 inputs
  for(icnt = 4, tcnt = 0; icnt < 12; icnt++, tcnt += 2)
     reftemps[tcnt] = reftemps[tcnt + 1] = (double)(adcinternals[icnt] - lg0) * 1000 / lg10;

  return reftemps[cnl];
}


// Get one external ADC (integated)
//  iom: module number
__declspec (dllexport) double __stdcall getadc(int mm, int iom, int cnl)
{
  void *x;                // Transaction object.

  if(cnl < 0 || cnl >= NUM_ADC)
    cnl = 0;
  x = S26_SchedOpen(mm, 1);
  S26_Sched2608_GetAins(x, iom, adcval, 1);
  io_exec(x);

  return adcval[cnl];
}


// Set one type and store to intype[] array
//  iom: module number
__declspec (dllexport) void __stdcall setintype(int mm, int iom, int cnl, int type)
{
  void *x;                // Transaction object.

  if(cnl < 0 || cnl >= NUM_ADC)
    cnl = 0;
  intype[cnl] = type;
  x = S26_SchedOpen(mm, 1);
  S26_Sched2608_SetAinTypes(x, iom, intype);
  io_exec(x);
}


// printable names of input modes
static char *mnames[] =
  {
  "LG val",
  "HG val",
  "10V",
  "100mV",
  "TC B",
  "TC C",
  "TC E",
  "TC J",
  "TC K",
  "TC N",
  "TC R",
  "TC S",
  "TC T"
  };

#define NTYPES  (sizeof(mnames) / sizeof(mnames[0]))    // number of input types

// Get ascii name of given input type
__declspec (dllexport) char *__stdcall getintype(int type)
{
  if(type < 0 || type >= NTYPES)
    return "Invalid type";
  return mnames[type];
}

// Set one DAC
//  iom: module number
//  dac: DAC number
//  val: DAC value
__declspec (dllexport) void __stdcall setdac(int mm, int iom, int dac, double val)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  S26_Sched2608_SetAout(x, iom, dac, val);
  io_exec(x);
}


// Set 50/60Hz standard
//  iom: module number
//  f50hz: 0 - 60Hz; 1 - 50Hz
__declspec (dllexport) void __stdcall setfstandard(int mm, int iom, int f50hz)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  S26_Sched2608_SetLineFreq(x, iom, f50hz);
  io_exec(x);
}


// Get last error message
// Clear error status if required
__declspec (dllexport) char *__stdcall getlasterror(int clear)
{
  static char err[256];

  strcpy(err, errstr);
  if(clear)
    strcpy(errstr, "Ok");
  return err;
}



/////////////////////////////////////////////////////////////////
// internal functions
/////////////////////////////////////////////////////////////////

// Open and init main DLL
static int dllinit(void)
{
  int err;

  // open DLL
  #if DEBUG
  // local directory
  //err = S26_DLLOpen("d:\\ntprog\\windrv\\2600\\labview\\2600demo", 1);
  err = S26_DLLOpen("c:\\2600\\labview", 1);
  #else
  // some common directory (windows\system32)
  err = S26_DLLOpen(NULL, 1);
  #endif

  if(err)
    {
    error("2600 DLL open error #%d", err);
    return(-1);
    }

  return(0);
}


#define BSWAP(c)    (((c) << 8) & 0xff00 | ((c) >> 8) & 0xff) // swap bytes in short
#define MSEC_WDOG 1000          // MM watchdog interval, in msec.



// Close main DLL
static void dllclose(void)
{
  S26_DriverClose();
}


// Find a module
//  id: module ID
//  from: number of module to start from
static int findmodule(int mm, int id, int from)
{
  int err;
  int iom;
  unsigned short num;
  unsigned short ioms[16];
  unsigned char sts[16];

  if(from < 0)
    {
    err = S26_RegisterAllIoms(mm, 1000, &num, ioms, sts, 1);
    if(err && err != 0x00000c0d)
      return(-1);
    if(err == 0x00000c0d)
      error("Calibration values out of tolerance: defaults applied");
    from = 0;
    }
  // Count the connected IOMs, stop on first board found.
  for(iom = from; iom < 16; iom++)
    {
    if(ioms[iom] == id)
      {
      return(iom);
      }
    }
  return(-1);
}


// Get all internal ADCs to adcinternals
//  iom: module number
static void getinternals(int mm, int iom)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  S26_Sched2608_GetCalData(x, iom, adcinternals);
  io_exec(x);
}



// Error message
static void error(char *fmt, ... )
{
  va_list argptr;

  va_start(argptr, fmt);
  vsprintf(errstr, fmt, argptr);
  va_end(argptr);
  #if DEBUG
    {
    FILE *f;

    if((f = fopen("c:\\2608vidll.log", "a")) == NULL)
      return;
    fprintf(f, "%s\n", errstr);
    fclose(f);
    }
  #endif

}


// Execute all scheduled i/o.
// Returns zero if successful.
static int io_exec(void *x)
{
  char status[16];		// Iom status info.
  GWERR err;

  // Execute the scheduled i/o.  Report error if one was detected.
  err = S26_SchedExecute(x, 100, status );
  if(err)
    error("Connection error 0x%08x", err);
  return(err);
}



